function Test-Administrator { Return (New-Object System.Security.Principal.WindowsPrincipal ( [System.Security.Principal.WindowsIdentity]::GetCurrent() ) ).IsInRole([System.Security.Principal.WindowsBuiltInRole]::Administrator) }

If ( -Not ( Test-Administrator ) ) { throw [System.UnauthorizedAccessException] "Garmin MapSoure Registration: Registration requires administrative privileges." }

If ( Test-Path -Path "HKLM:\SOFTWARE\Garmin\Mapsource" ) { ${MAP_SOURCE_BASE_PATH} = "HKLM:\SOFTWARE\Garmin\Mapsource\Families" }
ElseIf ( Test-Path -Path "HKLM:\SOFTWARE\Wow6432Node\Garmin\Mapsource" ) { ${MAP_SOURCE_BASE_PATH} = "HKLM:\SOFTWARE\Wow6432Node\Garmin\Mapsource\Families" }
Else { throw [System.IO.FileNotFoundException] "Garmin MapSoure Registration: Registration requires either Garmin Mapsource or Basecamp to be installed." }

$MAP_NAME = 'Sachsen'
$MAP_ID = '7008'

@("${MAP_NAME}.img", "${MAP_NAME}.tdb", "${MAP_NAME}.mdx", "${MAP_NAME}_mdr.img") | ForEach-Object { If ( -Not ( Test-Path -Path ( Join-Path -Path . -ChildPath $_ ) -PathType Leaf ) ) { throw [System.IO.FileNotFoundException] $_ } }

$Title = "Garmin MapSoure Registration for '${MAP_NAME} [${MAP_ID}]'"
$Message = "This process will can register or unregister the present map '${MAP_NAME} [${MAP_ID}]' within this computer's registry. After registration the map will be available to use within Garmin Mapsource or Basecamp.`r`nPlease select how to proceed."

$REGISTER = New-Object System.Management.Automation.Host.ChoiceDescription "&Register map", "Register map: The map will be registered within this computer's registry."
$UNREGISTER = New-Object System.Management.Automation.Host.ChoiceDescription "&Unregister map", "Unregister map: The map will be unregistered from this computer's registry."
$ABORT = New-Object System.Management.Automation.Host.ChoiceDescription "&Abort", "Abort: The process will be aborted."

$Options = [System.Management.Automation.Host.ChoiceDescription[]]($REGISTER, $UNREGISTER, $ABORT)

$SELECTION = $host.UI.PromptForChoice($Title, $Message, $Options, 0)

switch( $SELECTION )
{
0 {
New-Item -Path ( Join-Path -Path ${MAP_SOURCE_BASE_PATH} -ChildPath ${MAP_NAME} ) -Force | Out-Null
New-ItemProperty -Path ( Join-Path -Path ${MAP_SOURCE_BASE_PATH} -ChildPath ${MAP_NAME} ) -Name 'TYP' -PropertyType String -Value (Resolve-Path -Path ".\${MAP_ID}.typ") -Force | Out-Null
New-ItemProperty -Path ( Join-Path -Path ${MAP_SOURCE_BASE_PATH} -ChildPath ${MAP_NAME} ) -Name 'ID' -PropertyType Binary -Value ( [System.BitConverter]::GetBytes( [System.UInt16]::Parse(${MAP_ID}) ) ) -Force | Out-Null
New-ItemProperty -Path ( Join-Path -Path ${MAP_SOURCE_BASE_PATH} -ChildPath ${MAP_NAME} ) -Name 'IDX' -PropertyType String -Value (Resolve-Path -Path ".\${MAP_NAME}.mdx") -Force | Out-Null
New-ItemProperty -Path ( Join-Path -Path ${MAP_SOURCE_BASE_PATH} -ChildPath ${MAP_NAME} ) -Name 'MDR' -PropertyType String -Value (Resolve-Path -Path ".\${MAP_NAME}_mdr.img") -Force | Out-Null

New-Item -Path ( Join-Path -Path ${MAP_SOURCE_BASE_PATH} -ChildPath "${MAP_NAME}\1" ) -Force | Out-Null
New-ItemProperty -Path ( Join-Path -Path ${MAP_SOURCE_BASE_PATH} -ChildPath "${MAP_NAME}\1" ) -Name 'TDB' -PropertyType String -Value (Resolve-Path -Path ".\${MAP_NAME}.tdb") -Force | Out-Null
New-ItemProperty -Path ( Join-Path -Path ${MAP_SOURCE_BASE_PATH} -ChildPath "${MAP_NAME}\1" ) -Name 'LOC' -PropertyType String -Value (Resolve-Path -Path ".\" ) -Force | Out-Null
New-ItemProperty -Path ( Join-Path -Path ${MAP_SOURCE_BASE_PATH} -ChildPath "${MAP_NAME}\1" ) -Name 'BMAP' -PropertyType String -Value (Resolve-Path -Path ".\${MAP_NAME}.img") -Force | Out-Null

Write-Host -Object "The map '${MAP_NAME} [${MAP_ID}]' has been successfully registered within this computer's registry.`r`n"
}
1 {
If ( Test-Path -Path ( Join-Path -Path ${MAP_SOURCE_BASE_PATH} -ChildPath ${MAP_NAME} ) ) { Remove-Item -Path ( Join-Path -Path ${MAP_SOURCE_BASE_PATH} -ChildPath ${MAP_NAME} ) -Force -Recurse | Out-Null }

Write-Host -Object "The map '${MAP_NAME} [${MAP_ID}]' has been successfully unregistered from this computer's registry.`r`n"
}
2 {

}
}
